/**
 * Main application component.
 * Sets up the main layout using Header, Page, and Footer components.
 * Includes routing for different pages.
 */
import React, { lazy, Suspense, useMemo, useEffect } from 'react';
import { BrowserRouter as Router, Routes, Route, useLocation, useNavigate } from 'react-router-dom';
import Header from './components/Header';
import Page from './pages/Page';
import Footer from './components/Footer';
import SEO from './components/SEO';
import Analytics from './components/Analytics';
// Stagewise removed - development tool not needed in production

// Lazy load admin components for better performance
const DashboardPage = lazy(() => import('./pages/admin/DashboardPage'));
const AdminLayout = lazy(() => import('./components/admin/AdminLayout'));
const NewCasePage = lazy(() => import('./pages/admin/NewPage'));
const EditCasePage = lazy(() => import('./pages/admin/EditPage'));
const LoginPage = lazy(() => import('./pages/admin/LoginPage'));
const MediaManagerPage = lazy(() => import('./pages/admin/MediaManagerPage'));
const LayoutSettingsPage = lazy(() => import('./pages/admin/LayoutSettingsPage'));

import NotFoundPage from './pages/NotFoundPage';
import ErrorBoundary from './components/ErrorBoundary';
import { ToastProvider } from './contexts/ToastContext';
import { AuthProvider } from './contexts/AuthContext';
import { ThemeProvider } from './contexts/ThemeContext';
import { LanguageProvider } from './components/LanguageRouter';
import './App.css';

import { useLayoutSettings } from './hooks/useLayoutSettings';

// Memoized Header component to prevent unnecessary re-renders
const MemoizedHeader = React.memo(Header);

// Component to handle trailing slash redirects
const TrailingSlashRedirect: React.FC = () => {
  const location = useLocation();
  const navigate = useNavigate();

  useEffect(() => {
    // Check if URL has trailing slash (but not for root path)
    if (location.pathname !== '/' && location.pathname.endsWith('/')) {
      // Remove trailing slash and preserve search params
      const newPath = location.pathname.slice(0, -1);
      const newUrl = newPath + location.search + location.hash;
      console.log(`Redirecting from ${location.pathname} to ${newPath} (removing trailing slash)`);
      navigate(newUrl, { replace: true });
    }
  }, [location, navigate]);

  return null;
};

// Component to conditionally render Header based on route
const ConditionalHeader: React.FC = React.memo(() => {
  const location = useLocation();
  
  // Memoize the route type to prevent unnecessary re-renders
  const isAdminRoute = useMemo(() => {
    return location.pathname.startsWith('/admin');
  }, [location.pathname]);
  
  // Don't show header on admin routes
  if (isAdminRoute) {
    return null;
  }
  
  // Use memoized header component
  return <MemoizedHeader />;
});

// Memoized Footer component to prevent unnecessary re-renders
const MemoizedFooter = React.memo(Footer);

// Component to conditionally render Footer based on route
const ConditionalFooter: React.FC = React.memo(() => {
  const location = useLocation();
  
  // Memoize the route type to prevent unnecessary re-renders
  const isAdminRoute = useMemo(() => {
    return location.pathname.startsWith('/admin');
  }, [location.pathname]);
  
  // Don't show footer on admin routes
  if (isAdminRoute) {
    return null;
  }
  
  // Use memoized footer component
  return <MemoizedFooter />;
});

function App() {
  // Load global layout settings and apply them as CSS variables
  useLayoutSettings();
  return (
    <Router>
      <ThemeProvider>
        <AuthProvider>
          <ToastProvider>
            <LanguageProvider>
              <ErrorBoundary>
                <SEO />
                <Analytics 
                  measurementId={import.meta.env.VITE_GA_MEASUREMENT_ID || "G-5T68SH1TE5"}
                  cookieConsentRequired={true}
                  debugMode={false}
                />
                <TrailingSlashRedirect />
                <div className="app-container">
                  <ConditionalHeader />
                  <main className="main-content">
                    <ErrorBoundary>
                      <Routes>
                        {/* Public Routes - Dutch (default) */}
                        <Route path="/" element={<Page />} />
                        
                        {/* System pages */}
                        <Route path="/clients" element={<Page />} />
                        <Route path="/cases" element={<Page />} />
                        
                        {/* Legacy case routes - these will redirect via API */}
                        <Route path="/clients/:caseId" element={<Page />} />
                        <Route path="/cases/:caseId" element={<Page />} />
                        
                        {/* Public Routes - English */}
                        <Route path="/en" element={<Page />} />
                        <Route path="/en/clients" element={<Page />} />
                        <Route path="/en/cases" element={<Page />} />
                        <Route path="/en/clients/:caseId" element={<Page />} />
                        <Route path="/en/cases/:caseId" element={<Page />} />
                        <Route path="/en/:parentId/:childId" element={<Page />} />
                        <Route path="/en/:pageId" element={<Page />} />

                      {/* Admin Routes - Lazy loaded */}
                      <Route path="/admin/login" element={
                        <Suspense fallback={<div>Loading...</div>}>
                          <LoginPage />
                        </Suspense>
                      } />
                      <Route path="/admin" element={
                        <Suspense fallback={<div>Loading admin...</div>}>
                          <AdminLayout />
                        </Suspense>
                      }>
                        <Route index element={
                          <Suspense fallback={<div>Loading dashboard...</div>}>
                            <DashboardPage />
                          </Suspense>
                        } />
                        <Route path="dashboard" element={
                          <Suspense fallback={<div>Loading dashboard...</div>}>
                            <DashboardPage />
                          </Suspense>
                        } />
                        <Route path="cases/new" element={
                          <Suspense fallback={<div>Loading editor...</div>}>
                            <NewCasePage />
                          </Suspense>
                        } />
                        <Route path="cases/edit/:id" element={
                          <Suspense fallback={<div>Loading editor...</div>}>
                            <EditCasePage />
                          </Suspense>
                        } />
                        <Route path="pages/edit/:id" element={
                          <Suspense fallback={<div>Loading editor...</div>}>
                            <EditCasePage />
                          </Suspense>
                        } />
                        <Route path="media" element={
                          <Suspense fallback={<div>Loading media manager...</div>}>
                            <MediaManagerPage />
                          </Suspense>
                        } />
                        <Route path="layout-settings" element={
                          <Suspense fallback={<div>Loading layout settings...</div>}>
                            <LayoutSettingsPage />
                          </Suspense>
                        } />
                      </Route>

                      {/* Hierarchical pages - these need to come after specific routes */}
                      <Route path="/:parentId/:childId" element={<Page />} />
                      
                      {/* Single level custom pages - must be last before catch-all */}
                      <Route path="/:pageId" element={<Page />} />
                      
                      {/* Catch-all route for 404 */}
                      <Route path="*" element={<NotFoundPage />} />
                    </Routes>
                  </ErrorBoundary>
                </main>
                <ConditionalFooter />
              </div>
            </ErrorBoundary>
          </LanguageProvider>
          </ToastProvider>
        </AuthProvider>
      </ThemeProvider>
    </Router>
  );
}

export default App;
